/*
 * Test manual mapping of dataspace content.
 *
 * Copyright (C) 2020, 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <systypes/fcntl.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <fsclient/file.h>
#include <mem/memory_utils.h>

#include "dataspace_client.h"



int main(int argc, char *argv[])
{
  if (argc < 4)
  {
    printf("Need filename, step and sample size.\n");
    return 1;
  }

  /* Obtain filename and access parameters. */

  char *filename = argv[1];
  unsigned long step = atoi(argv[2]);
  unsigned long sample = atoi(argv[3]);

  /* Allocate a buffer for sampling from the file. */

  char buf[sample + 1];

  /* Obtain access to the filesystem. */

  l4_cap_idx_t server = l4re_env_get_cap("server");

  /* Invoke the open method to receive the file reference. */

  file_t file;
  long err = file_open(&file, filename, O_RDWR, server);

  if (err)
  {
    printf("Could not obtain file: %s\n", l4sys_errtostr(err));
    return 1;
  }

  /* A region of the file is mapped but not attached. */

  err = file_mmap_only(&file, 0, page(10), 0, 0);

  if (err)
  {
    printf("Could not map file region: %s\n", l4sys_errtostr(err));
    return 1;
  }

  /* Fix up the file data structure manually. */

  file.memory = (char *) 0x2000000;

  /* Explicitly map the file into this address space, without region mapper
     usage. The receive window base must be a multiple of its size. */

  client_Dataspace dataspace(file.ref);
  l4_snd_fpage_t region = {0, l4_fpage((l4_addr_t) file.memory, L4_PAGESHIFT + 4, 0)};

  printf("region = {%lx, {%lx, %d}}\n", region.snd_base, l4_fpage_memaddr(region.fpage), l4_fpage_size(region.fpage));

  err = dataspace.map(0, 0, L4_FPAGE_RO, &region);

  if (err)
  {
    printf("Could not invoke map successfully: %s\n", l4sys_errtostr(err));
    return 1;
  }

  for (unsigned long offset = 0; offset < file_populated_span(&file); offset += step)
  {
    unsigned long remaining = file_populated_span(&file) - offset;
    unsigned long sample_remaining = remaining < sample ? remaining : sample;

    printf("%ld bytes from %p...\n", sample_remaining, (file.memory + offset));
    strncpy(buf, (file.memory + offset), sample_remaining);
    buf[sample_remaining] = '\0';
    printf("%s\n", buf);
  }

  printf("End of test.\n");
  return 0;
}

// vim: tabstop=2 expandtab shiftwidth=2
